from .fragments import ASSET_FRAGMENT


GQL_APPEND_MANY_TO_DATASET = f'''
mutation(
    $projectID: ID!,
    $contentArray: [String!],
    $externalIDArray: [String!],
    $isHoneypotArray: [Boolean!],
    $statusArray: [Status!],
    $jsonContentArray: [String!],
    $jsonMetadataArray: [String!]) {{
  data: appendManyToDataset(
    projectID: $projectID,
    contentArray: $contentArray,
    externalIDArray: $externalIDArray,
    isHoneypotArray: $isHoneypotArray,
    statusArray: $statusArray,
    jsonContentArray: $jsonContentArray,
    jsonMetadataArray: $jsonMetadataArray
  ) {{
    {ASSET_FRAGMENT}
  }}
}}
'''


GQL_UPDATE_PROPERTIES_IN_ASSETS = f'''
mutation(
    $whereArray: [AssetWhere!]!
    $dataArray: [AssetData!]!
) {{
  data: updatePropertiesInAssets(
    where: $whereArray,
    data: $dataArray
  ) {{
    {ASSET_FRAGMENT}
  }}
}}
'''

GQL_DELETE_MANY_FROM_DATASET = f'''
mutation($assetIDs: [ID!]) {{
  data: deleteManyFromDataset(assetIDs: $assetIDs) {{
    {ASSET_FRAGMENT}
  }}
}}
'''



from typing import List, Optional

from typeguard import typechecked

from ...helpers import Compatible, deprecate, format_result, fragment_builder
from .queries import gql_issues
from ...types import Issue as IssueType


class QueriesIssue:

    def __init__(self, auth):
        """
        Initializes the subclass

        Parameters
        ----------
        - auth : KiliAuth object
        """
        self.auth = auth

    @Compatible(['v1', 'v2'])
    @typechecked
    def issues(self,
               fields: Optional[list] = [
                       'id',
                       'createdAt',
                       'hasBeenSeen',
                       'issueNumber',
                       'status',
                       'type'],
               first: Optional[int] = 100,
               project_id: Optional[str] = None,
               skip: Optional[int] = 0):
        """
        Get an array of issues given a set of criteria

        Parameters
        ----------
        - project_id : str, optional (default = None)
            Project ID the issue belongs to.

        Returns
        -------
        - a result object which contains the query if it was successful, or an error message else.

        Examples
        -------
        >>> # List all issues of a project and their authors
        >>> kili.issues(project_id=project_id, fields=['author.email'])
        """
        variables = {
            'where': {
                'project': {
                    'id': project_id,
                },
            },
            'skip': skip,
            'first': first,
        }
        GQL_ISSUES = gql_issues(fragment_builder(fields, IssueType))
        result = self.auth.client.execute(GQL_ISSUES, variables)
        return format_result('data', result)



def gql_project_version(fragment):
    return f'''
query ($where: ProjectVersionWhere!, $first: PageSize!, $skip: Int!) {{
  data: projectVersions(where: $where, first: $first, skip: $skip) {{
    {fragment}
  }}
}}
'''


GQL_PROJECT_VERSION_COUNT = f'''
query($where: ProjectVersionWhere!) {{
  data: countProjectVersions(where: $where)
}}
'''



def gql_issues(fragment):
    return(f'''
query ($where: IssueWhere!, $first: PageSize!, $skip: Int!) {{
  data: issues(where: $where, first: $first, skip: $skip) {{
    {fragment}
  }}
}}
''')


GQL_LABELS_COUNT = f'''
query($where: IssueWhere!) {{
  data: countIssues(where: $where)
}}
'''



from typing import List, Optional

from typeguard import typechecked

from ...helpers import Compatible, format_result, fragment_builder
from .queries import gql_project_version, GQL_PROJECT_VERSION_COUNT
from ...types import ProjectVersion as ProjectVersionType

class QueriesProjectVersion:

    def __init__(self, auth):
        """
        Initializes the subclass

        Parameters
        ----------
        - auth : KiliAuth object
        """
        self.auth = auth

    @Compatible(['v2'])
    @typechecked
    def project_version(self,
                      first: Optional[int] = 100,
                      skip: Optional[int] = 0,
                      fields: List[str] = ['createdAt', 'id', 'content', 'name', 'project', 'projectId'],
                      project_id: str = None,
                      ):
        """
        Get an array of project version given a set of constraints

        Parameters
        ----------
        - fields : list of string, optional (default = ['createdAt', 'id', 'content', 'name', 'project'])
            All the fields to request among the possible fields for the project versions
            See [the documentation](https://cloud.kili-technology.com/docs/python-graphql-api/graphql-api/#projectVersions) for all possible fields.
        - first : int (default = 100)
            Optional, Number of project versions to query
        - project_id : string (default = '')
            Filter on Id of project
        - skip : int (default = 0)
            Optional, number of project versions to skip (they are ordered by their date of creation, first to last).

        Returns
        -------
        - a result object which contains the query if it was successful, or an error message else.
        """
        formatted_first = first if first else 100
        variables = {
            'where': {
                'projectId': project_id,
            },
            'skip': skip,
            'first': formatted_first,
        }
        GQL_PROJECT_VERSION = gql_project_version(
            fragment_builder(fields, ProjectVersionType))
        result = self.auth.client.execute(GQL_PROJECT_VERSION, variables)
        return format_result('data', result)

    @Compatible(['v2'])
    @typechecked
    def count_project_versions(self,
                            project_id: str):
        """
        Count the number of project versions

        Parameters
        ----------
        - project_id :
            Filter on ID of project

        Returns
        -------
        - the number of project versions with the parameters provided
        """
        variables = {
            'where': { 'projectId' : project_id
            },
        }
        result = self.auth.client.execute(GQL_PROJECT_VERSION_COUNT, variables)
        count = format_result('data', result)
        return count
